# 1. Encodes high-functionality python SDK for streaming data from senspots into .txt, .csv, .xml, etc.
# 2. Streams data from USB sink serial port to a local sqlite db
# 3. ETL functions to push locally streamed data to MySQL db on resensys.net
# 4. Option to open a streaming window to view all present streaming devices

